@@ -2,18 +2,11 @@
    FreeWheeling
    ************
 
-   SLOWLY
-
-   GENTLY
-
-   PATIENTLY
-
-   DEEP
-
-   DEEP
-
-   COMMUNION
-
+   What is music,
+   if it is not shared in community,
+   held in friendship,
+   alive and breathing,
+   soil and soul?
 
    THANKS & PRAISE
 
@@ -42,9 +35,10 @@
 #include "fweelin_core.h"
 #include "fweelin_core_dsp.h"
 
+#if 1
 int main (int argc, char *argv[]) {
   Fweelin flo;
-
+  
   printf("FreeWheeling %s\n",VERSION);
   printf("Brought to you by a grant from Mother Earth.\n\n");
 
@@ -52,6 +46,9 @@ int main (int argc, char *argv[]) {
     flo.go();
   else
     printf("Error starting FreeWheeling!");
+  
+  return 0;
 }
+#endif
 
 // Improvisation is loving what is.
