@@ -5,12 +5,32 @@
 #include <vorbis/vorbisfile.h>
 #include <vorbis/vorbisenc.h>
 
+#ifdef __MACOSX__
+#include <Sndfile/sndfile.h>
+#else
+#include <sndfile.h>
+#endif
+
 #include "fweelin_audioio.h"
 #include "fweelin_mem.h"
 #include "fweelin_event.h"
 
+// Types of audio codecs supported
+typedef enum {UNKNOWN = -1,
+              FIRST_FORMAT = 0, 
+	      VORBIS = 0,
+	      WAV = 1,
+	      FLAC = 2,
+	      AU = 3,
+	      END_OF_FORMATS = 4} codec;
+
+#ifndef MIN
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
+#endif
+
+#ifndef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
+#endif
 
 class Fweelin;
 class Loop;
@@ -344,54 +364,95 @@ class GrowChainManager : public ManagedChain {
   virtual int Manage();
 };
 
-class VorbisEncoder {
+// Base class for different types of file encoders
+class iFileEncoder {
+ public:
+  iFileEncoder (Fweelin *app, char stereo);
+  virtual ~iFileEncoder() {};
+
+  // Tell the encoder to dump to this file we have just opened
+  // This writes the files header information 
+  // Returns nonzero on error.
+  virtual int SetupFileForWriting (FILE *file) = 0;
+
+  // Get the samples from the relevant buffer (ibuf) and write them to file,
+  // (startframe) tells us what index to start copying from the input buffer,
+  // (numframes) tells us how many frames to encode.
+  // Returns the number of frames written.
+  virtual long int WriteSamplesToDisk (sample_t **ibuf, nframes_t startframe, nframes_t numframes) = 0;
+
+  // Stop encoding and finish up- but don't close output file!
+  virtual void PrepareFileForClosing () = 0;
+
+  // Vorbis in particular needs to a bit of scaling down to avoid clipping... 
+  // any simular processing should be put here
+  virtual void Preprocess (sample_t *l, sample_t *r, nframes_t len) = 0;
+
+  inline char IsStereo() { return stereo; };
+
+ protected:
+
+  Fweelin *app;
+  char stereo;
+  FILE *outfd;
+};
+
+class SndFileEncoder : public iFileEncoder {
+ public:
+  // Encoder for libSndfile formats- at the moment, WAV, AU and FLAC.
+  // Maxframes is maximum number of frames to write in one call to 
+  // WriteSamplesToDisk
+  // Codec type is one of those values.
+  // We also specify wether we are using Mono or Stereo Encoding
+  SndFileEncoder (Fweelin *app, nframes_t maxframes, char stereo, codec type);
+  ~SndFileEncoder() {
+    if (tbuf != 0)
+      delete[] tbuf;
+  };
+  
+  int SetupFileForWriting (FILE *file);
+  long int WriteSamplesToDisk (sample_t **ibuf, nframes_t startframe, nframes_t numframes);
+  void PrepareFileForClosing (void);
+  void Preprocess (sample_t *l, sample_t *r, nframes_t len) {};
+
+ private:
+
+  codec filetype;
+  int samplesize;
+  SF_INFO sfinfo;
+  SNDFILE *sndoutfd;
+  float *tbuf; // Temporary audio buffer for saving
+};
+
+class VorbisEncoder : public iFileEncoder {
  public:
 
   // Vorbis encoder library init/end are done in constructor/destructor
   // We specify stereo or mono encoding
-  VorbisEncoder(Fweelin *app, char stereo);
-  ~VorbisEncoder();
+  VorbisEncoder (Fweelin *app, char stereo);
+  virtual ~VorbisEncoder();
 
-  // Are we encoding in stereo? 
-  inline char IsStereo() { return stereo; };
-
-  // Returns vorbis encoder's analysis buffers for len frames
-  // Depending on mono/stereo it is an array of 1/2 by len samples
-  float **GetAnalysisBuffer(nframes_t len) { 
-    return vorbis_analysis_buffer(&vd,len);
-  };
-
-  // Tell vorbis we wrote some samples to its analysis buffer
-  void WroteToBuffer(nframes_t len) {
-    vorbis_analysis_wrote(&vd,len);
-  };
-
-  // Runs the encoder, returns number of bytes written
-  long int Encode();
-
-  // Stop encoding and finish up- but don't close output file!
-  void Stop() {
+  int SetupFileForWriting (FILE *file);
+  long int WriteSamplesToDisk (sample_t **ibuf, nframes_t startframe, nframes_t numframes);
+  void PrepareFileForClosing (void) {
     // Tell vorbis we are done with the stream
     vorbis_analysis_wrote(&vd,0);
-
     // Encode any remaining stuff
     Encode();
   };
-
-  // Tell the encoder to dump to this file we have just opened
-  // This writes the vorbis header
-  void DumpToFile(FILE *out);
+  void Preprocess (sample_t *l, sample_t *r, nframes_t len);
 
  private:
 
-  // Fweelin
-  Fweelin *app; 
-  
-  // Stereo encoding?
-  char stereo;
+  // Returns vorbis encoder's analysis buffers for len frames
+  // Depending on mono/stereo it is an array of 1/2 by len samples
+  float **GetAnalysisBuffer(nframes_t len) { return vorbis_analysis_buffer(&vd,len); };
 
-  // Output to this file
-  FILE *outfd;
+  // Tell vorbis we wrote some samples to its analysis buffer
+  void WroteToBuffer(nframes_t len) { vorbis_analysis_wrote(&vd,len); };
+
+  // Runs the encoder, returns number of bytes written
+  long int Encode();
 
   // Vorbis encoder stuff
   ogg_stream_state os; /* take physical pages, weld into a logical
@@ -407,32 +468,27 @@ class VorbisEncoder {
   vorbis_block     vb; /* local working space for packet->PCM decode */
 };
 
-class VorbisDecoder {
- public:
 
-  // Vorbis library init/end are done in constructor/destructor
-  VorbisDecoder(Fweelin *app);
-  ~VorbisDecoder();
+class iFileDecoder {
+ public:
+  iFileDecoder (Fweelin *app);
+  virtual ~iFileDecoder () {};
+  
+  // Tell the decoder to decode from a given file
+  // This reads the files header
+  virtual int ReadFromFile(FILE *in, nframes_t rbuf_len) = 0;
+  // Decode a maximum of max_len samples and return buffers in pcm_channels
+  // (format as per VorbisFile ov_read_float)
+  // Returns # of samples read, 0 if EOF, <0 if error
+  virtual int ReadSamples(AudioBlockIterator *i, nframes_t max_len) = 0;
+  // Stop decoding and finish up-- and close the input file!
+  // ??? -- old note? FIXME workaround for memory managment issue
+  virtual void Stop() = 0;
 
   // Are we decoding in stereo? 
   inline char IsStereo() { return stereo; };
 
-  // Decode a maximum of max_len samples and return buffers in pcm_channels
-  // (format as per VorbisFile ov_read_float)
-  // Returns # of samples read, 0 if EOF, <0 if error
-  int Decode(float ***pcm_channels, nframes_t max_len);
-
-  // Stop decoding and finish up-- and close the input file!
-  void Stop() {
-    // Tell vorbis we are done with the stream
-    ov_clear(&vf);
-  };
-
-  // Tell the decoder to decode from a given file
-  // This reads the vorbis header
-  int ReadFromFile(FILE *in);
-
- private:
+ protected:
 
   // Fweelin
   Fweelin *app; 
@@ -442,6 +498,53 @@ class VorbisDecoder {
 
   // Input from this file
   FILE *infd;
+};
+
+class SndFileDecoder : public iFileDecoder {
+ public:
+  SndFileDecoder(Fweelin *app, codec type);
+  ~SndFileDecoder();
+  
+  int ReadFromFile (FILE *in, nframes_t rbuf_len);
+//   int Decode (float **pcm_channels, nframes_t max_len);
+  int ReadSamples(AudioBlockIterator *it, nframes_t max_len);
+  void Stop();
+
+ private:
+
+  codec filetype;
+  int samplesize;
+  SF_INFO sfinfo;
+  SNDFILE *sfinfd;
+  float *obuf[2];
+  float *rbuf;
+};
+
+class VorbisDecoder : public iFileDecoder {
+ public:
+
+  // Vorbis library init/end are done in constructor/destructor
+  VorbisDecoder(Fweelin *app);
+  ~VorbisDecoder();
+
+  // Decode a maximum of max_len samples and return buffers in pcm_channels
+  // (format as per VorbisFile ov_read_float)
+  // Returns # of samples read, 0 if EOF, <0 if error
+  int ReadSamples(AudioBlockIterator *i, nframes_t max_len);
+//   int Decode(float **pcm_channels, nframes_t max_len);
+
+  // Stop decoding and finish up-- and close the input file!
+  void Stop() {
+    // Tell vorbis we are done with the stream
+    ov_clear(&vf);
+  };
+
+
+  // Tell the decoder to decode from a given file
+  // This reads the vorbis header
+  int ReadFromFile(FILE *in, nframes_t rbuf_len);
+
+ private:
 
   // Vorbis decoder stuff
   OggVorbis_File vf;
@@ -494,6 +597,8 @@ class BlockReadManager : public ManagedChain {
   virtual ~BlockReadManager();
 
   virtual Preallocated *NewInstance() { return ::new BlockReadManager(); };
+
+  void SetLoopType (codec looptype);
   
   // BlockReadManager's RefDeleted may block to avoid block data being
   // deleted while we are still encoding- we finish up our current decode pass
@@ -521,8 +626,9 @@ class BlockReadManager : public ManagedChain {
   FILE *in;                 // File to read from
   char smooth_end;          // Smooth end of loop into beginning?
   AutoReadControl *arc;     // A way to ask app what blocks to read
-  VorbisDecoder *dec;       // Decoder
+  iFileDecoder *dec;        // Decoder
   BlockManager *bmg; 
+  codec filetype;
 
   PeaksAvgsManager *pa_mgr; // Computer for peaks/avgs during load
   nframes_t peaksavgs_chunksize;
@@ -595,7 +701,7 @@ class BlockWriteManager : public ManagedChain {
   nframes_t len,          // Length of block to save
     pos;                  // Current save position
   AutoWriteControl *awc;  // A way to ask app what blocks to write
-  VorbisEncoder *enc;
+  iFileEncoder *enc;
   BlockManager *bmg; 
 
   pthread_mutex_t encode_lock;
