@@ -16,6 +16,8 @@
 #include "fweelin_fluidsynth.h"
 #include "fweelin_browser.h"
 
+const double ItemRenamer::BLINK_DELAY = 0.5;
+
 ItemRenamer::ItemRenamer (Fweelin *app, RenameCallback *cb, 
 			  char *oldname) : app(app), cb(cb) {
   if (oldname == 0)
@@ -434,6 +436,18 @@ void Browser::MoveTo(int adjust, int jumpadjust) {
 	   GetTypeName(GetType()));
   }
 
+  // Call browser virtual method for item browsed-
+  // derived types of browsers may need to do internal updates
+  ItemBrowsed(); 
+
+  // Send out 'item-browsed' event-- which can be bound to
+  // in the config file.
+  BrowserItemBrowsedEvent *bievt = (BrowserItemBrowsedEvent *) 
+    Event::GetEventByType(T_EV_BrowserItemBrowsed);  
+  bievt->browserid = id;
+  // printf("BROWSER: itembrowsed send %p id: %d\n",bievt,id);
+  app->getEMG()->BroadcastEventNow(bievt, this);
+
   if (cur != 0 && callback != 0)
     callback->ItemBrowsed(cur);
 
@@ -872,8 +886,10 @@ PatchBrowser::~PatchBrowser() {
   // Invoke our special ClearAllItems method
   ClearAllItems(); 
 
-  if (app != 0)
+  if (app != 0) {
     app->getEMG()->UnlistenEvent(this,0,T_EV_PatchBrowserMoveToBank);
+    app->getEMG()->UnlistenEvent(this,0,T_EV_PatchBrowserMoveToBankByIndex);
+  }
 };
 
 void PatchBrowser::ClearAllItems() {
@@ -900,6 +916,8 @@ void PatchBrowser::ClearAllItems() {
     curpb = curpb->next;
   };
   
+  // *** Do we need to clear all patchbanks too?
+
   UnlockBrowser();
 };
 
@@ -936,8 +954,39 @@ void PatchBrowser::PB_MoveTo (int direction) {
     first = pb_cur->first;
     cur = pb_cur->cur;
 
-    pb_port = (pb_cur->port == -1 ? 0 : pb_cur->port);
-    pb_chan = pb_cur->channel;
+    SetMIDIEcho();
+  }
+
+  UnlockBrowser();
+};
+
+void PatchBrowser::PB_MoveToIndex (int index) {
+  LockBrowser();
+  
+  if (xpand) {
+    xpand_lastactivity = mygettime();
+    xpanded = 1;
+  }
+
+  // Update pb_cur
+  if (pb_cur != 0) {
+    pb_cur->cur = cur;
+    pb_cur->first = first;
+  } else
+    pb_cur = pb_first;
+
+  // Count to right patchbank
+  pb_cur = pb_first;
+  for (int i = 0; i < index && pb_cur != 0; i++, pb_cur = pb_cur->next);
+  
+  if (pb_cur == 0)
+    pb_cur = pb_first;
+  else {
+    // Assign patch 'cur' and 'first' pointers based on this patch bank
+    first = pb_cur->first;
+    cur = pb_cur->cur;
+
+    SetMIDIEcho();
   }
 
   UnlockBrowser();
@@ -946,6 +995,7 @@ void PatchBrowser::PB_MoveTo (int direction) {
 void PatchBrowser::Setup(Fweelin *a, BrowserCallback *c) {
   Browser::Setup(a,c);
   app->getEMG()->ListenEvent(this,0,T_EV_PatchBrowserMoveToBank);
+  app->getEMG()->ListenEvent(this,0,T_EV_PatchBrowserMoveToBankByIndex);
 };
 
 void PatchBrowser::ReceiveEvent(Event *ev, EventProducer *from) {
@@ -961,9 +1011,35 @@ void PatchBrowser::ReceiveEvent(Event *ev, EventProducer *from) {
     }
     break;
 
+  case T_EV_PatchBrowserMoveToBankByIndex :
+    {
+      PatchBrowserMoveToBankByIndexEvent *pbev = 
+	(PatchBrowserMoveToBankByIndexEvent *) ev;
+      
+      PB_MoveToIndex(pbev->index);
+      if (CRITTERS)
+	printf("PATCH BROWSER: Received PatchBrowserMoveToBankByIndex "
+	       "(index: %d)\n", pbev->index);
+    }
+    break;
+
   default:
     // Let base class handle it
     Browser::ReceiveEvent(ev,from);
     break;
   }
 };
+
+// Sets the right MIDI port(s) and channel(s) for echo based on this
+// patches' settings
+void PatchBrowser::SetMIDIEcho() {
+  /* printf("pb cur: %p port: %d\n",pb_cur,pb_cur->port);
+  printf("cur: %p\n",cur);
+  printf("app: %p\n",app);
+  printf("getmidi: %p\n",app->getMIDI()); */
+
+  if (app != 0 && app->getMIDI() != 0)
+    app->getMIDI()->SetMIDIEcho((pb_cur == 0 ? 0 : pb_cur->port),
+				(cur != 0 && cur->GetType() == B_Patch ?
+				 (PatchItem *) cur : 0));
+};
