@@ -13,6 +13,7 @@
 // It is precise action,
 // Energy conservation.
 
+#include <sys/stat.h>
 #include <sys/time.h>
 
 #include <stdio.h>
@@ -35,6 +36,13 @@
 
 // *********** CONFIG
 
+#ifdef __MACOSX__
+#import <sys/param.h> /* for MAXPATHLEN */
+
+char FWEELIN_datadir[MAXPATHLEN];
+char *FWEELIN_DATADIR = FWEELIN_datadir;
+#endif
+
 const char CfgMathOperation::operators[] = {'/', '*', '+', '-'};
 const int CfgMathOperation::numops = 4;
 
@@ -1314,7 +1322,6 @@ void InputMatrix::ReceiveEvent(Event *ev, EventProducer *from) {
       
       while (match != 0) {
 	// So we have a binding.. trigger the bound event!
-	
 	Event *shot = (Event *) match->boundproto->RTNew();
 	if (shot == 0) 
 	  printf("CONFIG: WARNING: Can't send event- RTNew() failed\n");
@@ -1337,7 +1344,8 @@ void InputMatrix::ReceiveEvent(Event *ev, EventProducer *from) {
 	if (echo == 0) 
 	  printf("CONFIG: WARNING: Can't send event- RTNew() failed\n");
 	else {
-	  *echo = *ev; // Copy from incoming input
+	  *echo = *ev;    // Copy from incoming input
+	  echo->echo = 1; // Set echo flag
 	  app->getEMG()->BroadcastEventNow(echo, this);    
 	}
       }
@@ -1359,18 +1367,37 @@ void InputMatrix::ReceiveEvent(Event *ev, EventProducer *from) {
 	if (sv->var == 0 || sv->var->IsSystemVariable())
 	  printf(" SetVariableEvent: Invalid variable!\n");
 	else {
-	  if (CRITTERS) {
-	    printf("CONFIG: SetVariable: ");
-	    sv->var->Print();
-	    printf(" -> ");
-	    
-	    sv->var->SetFrom(sv->value);
-	    
-	    sv->var->Print();
-	    printf("\n");
+	  // Check if variable change is within maxjump
+	  char goset = 1;
+	  if (sv->maxjumpcheck) {
+	    // Maxjump check enabled
+	    // Compute change in variable
+	    UserVariable jump = sv->var->GetDelta(sv->value);
+	    if (jump > sv->maxjump) 
+	      goset = 0;  // Don't set variable
+	  }
+	  
+	  if (goset) {
+	    if (CRITTERS) {
+	      printf("CONFIG: SetVariable: ");
+	      sv->var->Print();
+	      printf(" -> ");
+	      
+	      sv->var->SetFrom(sv->value);
+	      
+	      sv->var->Print();
+	      printf("\n");
+	    }
+	    else
+	      sv->var->SetFrom(sv->value);
+	  } else {
+	    // Don't set variable
+	    if (CRITTERS) {
+	      printf("CONFIG: SetVariable: ");
+	      sv->var->Print();
+	      printf(" -> [NOT SET- Max jump exceeded!]\n");	      
+	    }
 	  }
-	  else
-	    sv->var->SetFrom(sv->value);
 	}
       }
       break;
@@ -1471,8 +1498,17 @@ void FloConfig::ConfigureGeneral(xmlDocPtr doc, xmlNode *gen) {
 	printf("CONFIG: Starting with %d triggers.\n",num_triggers);
       } else if ((n = xmlGetProp(cur_node, 
 				 (const xmlChar *)"librarypath")) != 0) {
-	librarypath = new char[xmlStrlen(n)+1];
-	strcpy(librarypath,(char *) n);
+	if (xmlStrchr(n,'~') == n) {
+	  // Reference to home dir
+	  char *homedir = getenv("HOME");
+	  librarypath = new char[strlen(homedir)+xmlStrlen(n)+1];
+	  strcpy(librarypath,homedir);
+	  strcat(librarypath,(char *) &n[1]);
+	} else {
+	  librarypath = new char[xmlStrlen(n)+1];
+	  strcpy(librarypath,(char *) n);
+	}
+
 	char *ptr = strrchr(librarypath,'/');
 	if (ptr != 0 && strlen(ptr) == 1) // Trailing backslash?
 	  // Remove
@@ -1516,12 +1552,52 @@ void FloConfig::ConfigureGeneral(xmlDocPtr doc, xmlNode *gen) {
 	  limiterreleaserate = 0.000020;
 	printf("CONFIG: Limiter release rate set to %.2f%%.\n",
 	       limiterreleaserate*100);
+      } else if ((n = xmlGetProp(cur_node,
+                                 (const xmlChar *)"loopoutformat")) != 0) {
+	loopoutformat = GetCodecFromName((const char *) n);
+	if (loopoutformat == UNKNOWN)
+	  printf(FWEELIN_ERROR_COLOR_ON
+		 "CONFIG: Invalid loop output format: %s\n"
+		 FWEELIN_ERROR_COLOR_OFF,n);
+	else
+	  printf("CONFIG: Loop out format is: %s\n", GetCodecName(loopoutformat));
+      } else if ((n = xmlGetProp(cur_node,
+				 (const xmlChar *)"streamoutformat")) != 0) {
+	streamoutformat = GetCodecFromName((const char *) n);
+	if (streamoutformat == UNKNOWN)
+	  printf(FWEELIN_ERROR_COLOR_ON
+		 "CONFIG: Invalid stream output format: %s\n"
+		 FWEELIN_ERROR_COLOR_OFF,n);
+	else
+	  printf("CONFIG: Stream out format is: %s\n", 
+		 GetCodecName(streamoutformat));
+      } else if ((n = xmlGetProp(cur_node, 
+				 (const xmlChar *)"oggquality")) != 0) {
+	float q = atof((char *) n);
+	if (q <= 0.0)
+	  printf(FWEELIN_ERROR_COLOR_ON
+		 "CONFIG: Invalid OGG quality: %f\n"
+		 FWEELIN_ERROR_COLOR_OFF,q);
+	else {
+	  vorbis_encode_quality = q;
+	  printf("CONFIG: OGG Quality: %f\n",vorbis_encode_quality);
+	}
       } else if ((n = xmlGetProp(cur_node, 
 				 (const xmlChar *)"midiouts")) != 0) {
 	midiouts = atoi((char *) n);
 	if (midiouts < 1)
 	  midiouts = 1;
 	printf("CONFIG: Config sets %d MIDI outputs.\n",midiouts);
+      } else if ((n = xmlGetProp(cur_node,
+				 (const xmlChar *)"midisyncouts")) != 0) {
+	msouts = ExtractArrayInt((char *)n, &msnumouts);
+	printf("CONFIG: Set %d MIDI sync outputs: ",msnumouts);
+	for (int i = 0; i < msnumouts; i++) {
+	  printf("%d ",msouts[i]);
+	  msouts[i]--;	// Adjust so that 0 is the first output
+	}
+	
+	printf("\n");
       } else if ((n = xmlGetProp(cur_node, 
 				 (const xmlChar *)"externalaudioinputs")) != 0) {
 	// # of inputs
@@ -1646,9 +1722,9 @@ char FloConfig::IsStereoMaster() {
   return 0;
 };
 
-// Extracts a comma separated array of floats from the given string-
-// returns size of array in 'size'
-float *FloConfig::ExtractArray(char *n, int *size) {
+// Extracts an array of floats (delimited by character delim_char)
+// from the given string- returns size of array in 'size'
+float *FloConfig::ExtractArray(char *n, int *size, char delim_char) {
   char buf[255];
   strncpy(buf,n,254);
   buf[254] = '\0';
@@ -1657,7 +1733,7 @@ float *FloConfig::ExtractArray(char *n, int *size) {
   *size = 0;
   while (delim != 0) {
     (*size)++;
-    delim = strchr(delim+1,',');
+    delim = strchr(delim+1,delim_char);
   }
 
   if (*size == 0)
@@ -1668,7 +1744,7 @@ float *FloConfig::ExtractArray(char *n, int *size) {
   delim = buf;
   int i = 0;
   while (i < *size) {
-    char *nd = strchr(delim,',');
+    char *nd = strchr(delim,delim_char);
     if (nd != 0) {
       *nd = '\0';
       nd++;
@@ -1682,6 +1758,40 @@ float *FloConfig::ExtractArray(char *n, int *size) {
   return array;
 };
 
+int *FloConfig::ExtractArrayInt(char *n, int *size, char delim_char) {
+  char buf[255];
+  strncpy(buf,n,254);
+  buf[254] = '\0';
+  
+  char *delim = buf;
+  *size = 0;
+  while (delim != 0) {
+    (*size)++;
+    delim = strchr(delim+1,delim_char);
+  }
+  
+  if (*size == 0)
+    return 0;
+  
+  int *array = new int[*size];
+  
+  delim = buf;
+  int i = 0;
+  while (i < *size) {
+    char *nd = strchr(delim,delim_char);
+    if (nd != 0) {
+      *nd = '\0';
+      nd++;
+    }
+    //printf("%d: %s\n",i,delim);
+    
+    array[i++] = atoi(delim);
+    delim = nd;
+  }
+  
+  return array;
+};
+
 void FloConfig::ConfigureElement(xmlDocPtr doc, xmlNode *elemn, 
 				 FloLayoutElement *elem, float xscale,
 				 float yscale) {
@@ -1842,28 +1952,63 @@ void FloConfig::ConfigurePatchBanks(xmlNode *pb, PatchBrowser *br) {
     if ((!xmlStrcmp(cur_node->name, (const xmlChar *)"patchbank"))) {
       // Get MIDI port
       int pb_mport = 0;
-
       xmlChar *n = 0;
       if ((n = xmlGetProp(cur_node, (const xmlChar *)"midiport")) != 0) {
 	pb_mport = atoi((char *) n);
 	xmlFree(n);
       }
 
+      // Separate channels into distinct patchbanks?
+      char sepchan = 0;
+      if ((n = xmlGetProp(cur_node, 
+			  (const xmlChar *)"separatechannels")) != 0) {
+	sepchan = atoi((char *) n);
+	xmlFree(n);
+      }
+
+      // Suppress bank/program changes when changing patches? 
+      // (ie only change channel)
+      char suppresschg = 0;
+      if ((n = xmlGetProp(cur_node, 
+			  (const xmlChar *)"suppressprogramchanges")) != 0) {
+	suppresschg = atoi((char *) n);
+	xmlFree(n);
+      }
+
       xmlChar *pb_patches = 
 	xmlGetProp(cur_node, (const xmlChar *)"patches");
       if (pb_patches != 0) {
 	// PatchBank file defined
 	xmlDocPtr pb_doc;
 	char *homedir = getenv("HOME");
-	char buf[strlen(homedir) + strlen(FWEELIN_CONFIG_DIR) + 
+	char buf[strlen(FWEELIN_DATADIR) + 
+		 strlen(homedir) + strlen(FWEELIN_CONFIG_DIR) + 
 		 xmlStrlen(pb_patches) + 4];
+
+	// Try both ~/.fweelin and fweelin data dir
+	char notfound = 0;
 	sprintf(buf,"%s/%s/%s",homedir,FWEELIN_CONFIG_DIR,(char *) pb_patches);
+	struct stat st;
+
+	if (stat(buf,&st) != 0) {
+	  // Can't find it there. Try data dir
+	  sprintf(buf,"%s/%s",FWEELIN_DATADIR,(char *) pb_patches);
+	  if (stat(buf,&st) != 0) {
+	    // Not in either place!
+	    printf(FWEELIN_ERROR_COLOR_ON
+		   "\n*** INIT: ERROR: Can't find patches file '%s' in "
+		   "folders:\n\n%s/%s\n%s\n"
+		   FWEELIN_ERROR_COLOR_OFF,
+		   pb_patches,homedir,FWEELIN_CONFIG_DIR,FWEELIN_DATADIR);
+	    notfound = 1;
+	  }
+	}
 
 	// Load and parse
-	pb_doc = xmlParseFile(buf);
+	pb_doc = (notfound ? 0 : xmlParseFile(buf));
 	if (pb_doc == 0)
 	  printf(FWEELIN_ERROR_COLOR_ON
-		 "\n*** INIT: ERROR: Patches file '%s' does not exist.\n"
+		 "\n*** INIT: ERROR: Problem parsing patches file '%s'.\n"
 		 FWEELIN_ERROR_COLOR_OFF,
 		 pb_patches);
 	else {
@@ -1875,11 +2020,108 @@ void FloConfig::ConfigurePatchBanks(xmlNode *pb, PatchBrowser *br) {
 		   FWEELIN_ERROR_COLOR_OFF,
 		   pb_patches);
 	  else {
-	    int pb_curchan = -1,
+	    int p_curchan = -1,
 	      pcnt = 0;
 	    for (xmlNode *pb_curpatch = pb_root->children; pb_curpatch != NULL;
 		 pb_curpatch = pb_curpatch->next) {
-	      if ((!xmlStrcmp(pb_curpatch->name, (const xmlChar *)"patch"))) {
+	      if ((!xmlStrcmp(pb_curpatch->name, (const xmlChar *)"combi"))) {
+		// Combi (multi-zone multi-channel)
+
+		// First, count zones
+		int numzones = 0;
+		for (xmlNode *curzone = pb_curpatch->children; curzone != NULL;
+		     curzone = curzone->next)
+		  if ((!xmlStrcmp(curzone->name, (const xmlChar *)"zone")))
+		    numzones++;
+
+		printf("\nCOMBI: Numzones: %d\n",numzones);
+
+		// Setup combi
+		xmlChar *cname = 0;
+		cname = xmlGetProp(pb_curpatch,(const xmlChar *)"name");
+
+		if (pcnt == 0) {
+		  // First patch
+		  // Create new PatchBank
+		  br->PB_Add(new PatchBank(pb_mport,suppresschg));
+		}
+		  
+		PatchItem *pi = 
+		  new PatchItem(pcnt++,-1,-1,-1,(char *) cname);
+		pi->SetupZones(numzones);
+		br->AddItem(pi);
+		
+		if (pcnt % PatchBrowser::DIV_SPACING == 0)
+		  br->AddItem(new BrowserDivision());
+
+		if (cname != 0)
+		  xmlFree(cname);
+		
+		// For each zone
+		int curzone_idx = 0;
+		for (xmlNode *curzone = pb_curpatch->children; curzone != NULL;
+		     curzone = curzone->next) {
+		  if ((!xmlStrcmp(curzone->name, (const xmlChar *)"zone"))) {
+		    int kr_lo = 0,
+		      kr_hi = 0;
+		    if ((n = xmlGetProp(curzone, 
+					(const xmlChar *)"keyrange")) != 0) {
+		      int krs; 
+		      float *kr = ExtractArray((char *)n, &krs, '>');
+		      if (krs == 2) {
+			kr_lo = (int) kr[0];
+			kr_hi = (int) kr[1];
+		      }
+		      delete[] kr;
+		      xmlFree(n);
+		    }
+		    
+		    char mport_r = 0;
+		    int mport = pb_mport;
+		    xmlChar *n = 0;
+		    if ((n = xmlGetProp(curzone, 
+					(const xmlChar *)"midiport")) != 0) {
+		      mport_r = 1;
+		      mport = atoi((char *) n);
+		      xmlFree(n);
+		    }
+
+		    int bank = -1;
+		    if ((n = xmlGetProp(curzone, 
+					(const xmlChar *)"bank")) != 0) {
+		      bank = atoi((char *) n);
+		      xmlFree(n);
+		    }		
+		    
+		    int prog = -1;
+		    if ((n = xmlGetProp(curzone, 
+					(const xmlChar *)"program")) != 0) {
+		      prog = atoi((char *) n);
+		      xmlFree(n);
+		    }
+
+		    int chan = 0;
+		    if ((n = xmlGetProp(curzone,
+					(const xmlChar *)"channel")) != 0) {
+		      chan = atoi((char *) n);
+		      xmlFree(n);
+		    }
+
+		    printf("  ZONE [%d>%d]: midiport[%s]:%d "
+			   "bank: %d prog: %d chan: %d\n",
+			   kr_lo,kr_hi,
+			   (mport_r ? "REDIRECT" : "DEFAULT"),mport,
+			   bank,prog,chan);
+
+		    pi->GetZone(curzone_idx)->SetupZone(kr_lo,kr_hi,
+							mport_r,mport,
+							bank,prog,chan);
+		    curzone_idx++;
+		  }
+		}
+	      } else if ((!xmlStrcmp(pb_curpatch->name, 
+				     (const xmlChar *)"patch"))) {
+		// Single channel patch
 		int chan = 0;
 		if ((n = xmlGetProp(pb_curpatch,
 				    (const xmlChar *)"channel")) != 0) {
@@ -1904,16 +2146,21 @@ void FloConfig::ConfigurePatchBanks(xmlNode *pb, PatchBrowser *br) {
 		xmlChar *pname = 0;
 		if ((pname = xmlGetProp(pb_curpatch, 
 					(const xmlChar *)"name")) != 0) {
-		  if (chan != pb_curchan) {
-		    // New channel in list-- create new PatchBank
-		    br->PB_Add(new PatchBank(pb_mport,chan));
-		    pb_curchan = chan;
+		  if ((sepchan && chan != p_curchan) ||
+		      (!sepchan && pcnt == 0)) {
+		    // Separate channels (patchbank for each channel)-- 
+		    // or first patch 
+
+		    // Create new PatchBank
+		    br->PB_Add(new PatchBank(pb_mport,suppresschg));
+		    p_curchan = chan;
 		    pcnt = 0;
 		    //printf("new patchbank: mport: %d chan: %d\n",
 		    //	     pb_mport,chan);
 		  }
 		  
-		  br->AddItem(new PatchItem(pcnt++,bank,prog,(char *) pname));
+		  br->AddItem(new PatchItem(pcnt++,bank,prog,chan,
+					    (char *) pname));
 		  //printf("bank: %d prog: %d patch: '%s'\n",bank,prog,
 		  //	   (char *) pname);
 
@@ -2584,13 +2831,16 @@ FloConfig::~FloConfig()
 
 FloConfig::FloConfig(Fweelin *app) : im(app), 
   
-  ev_hook(0), librarypath(0), midiouts(1),
+  ev_hook(0), librarypath(0), midiouts(1), msnumouts(0), msouts(0),
 
   ms_inputs(0), extaudioins(0), 
 
   maxplayvol(5.0), maxlimitergain(1.0), limiterthreshhold(0.9), 
   limiterreleaserate(0.000020),
 
+  loopoutformat(VORBIS), streamoutformat(VORBIS),
+  vorbis_encode_quality(0.5),
+
   num_triggers(1024), vdelay(50000), showdebug(0), 
   layouts(0), fonts(0), displays(0), help(0),
 		
@@ -2608,6 +2858,14 @@ FloConfig::FloConfig(Fweelin *app) : im(app),
 
 // Parse configuration file, setup config
 void FloConfig::Parse() {  
+#ifdef __MACOSX__
+  // On Mac OS X, Set FWEELIN_DATADIR to bundle resource folder
+  
+  CFURLRef url = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
+  CFURLGetFileSystemRepresentation(url, true, (UInt8 *)FWEELIN_DATADIR, MAXPATHLEN);
+  CFRelease(url);
+#endif
+  
   // Setup event type table
   Event::SetupEventTypeTable(im.app->getMMG());
   xmlDocPtr doc;
@@ -2667,11 +2925,10 @@ void FloConfig::Parse() {
   }
 
   if (pause) {
-    printf("*** Please review the above change "
-	   "and press <ENTER> to continue.\n");
-    getchar();
+    printf("*** Please review the above change\n");
+    // getchar();
   }
-
+  
   doc = xmlParseFile(buf);
   if (doc == 0) {
     printf("INIT: Error parsing config file.\n");
@@ -2692,12 +2949,36 @@ void FloConfig::Parse() {
       if (ver == 0 || strcmp((char *) ver,VERSION)) {
 	printf("INIT: ERROR: Config file version \"%s\" does not match "
 	       "FreeWheeling version \"%s\"!\n\n",ver,VERSION);
+#if 0
+	// Choice
 	printf("Would you like to override your old config with the new file\n"
 	       "(warning: You'll lose any config changes you've made!) "
 	       "(y/N)? ");
  	char c = getchar();
-
+#endif
+	
+	// Force yes and backup- avoid console-based choice which appears like a crash
+	printf("*** Installing new configuration file. I will make a backup of the old one.\n");
+	
+	char c = 'Y';
 	if (c == 'Y' || c == 'y') {
+	  char tmp2[strlen(buf) + 16];
+	  int bCnt = 1;
+	  char go = 1;
+	  while (go) {
+	    sprintf(tmp2,"%s.backup.%d",buf,bCnt);
+	    if (stat(tmp2,&st) != 0)
+	      go = 0; // Free backup filename
+	    else
+	      bCnt++;
+	  }
+
+	  printf("Backing up your old configuration to: %s\n",tmp2);
+	  char tmp3[strlen(buf) + strlen(tmp2) + 10];
+	  sprintf(tmp3,"cp %s %s",buf,tmp2);
+	  printf("\nINIT: Executing: %s\n",tmp3);
+	  system(tmp3);
+	  
 	  char *homedir = getenv("HOME");
 	  char buf[strlen(FWEELIN_DATADIR) + strlen(homedir) + 
 		   strlen(FWEELIN_CONFIG_FILE) + 
@@ -2722,8 +3003,7 @@ void FloConfig::Parse() {
 		   "Did you run 'make install'?\n");
 	    exit(1);
 	  }
-	}
-	else {
+	} else {
 	  printf("\nYou can manually edit your config file to match the "
 		 "new one-- here is a diff of the two: \n\n");
 
