@@ -7,8 +7,20 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#define MAX_PULSES 10 // Maximum number of time pulses
-#define LAST_REC_COUNT 8 // Keep track of the last n indexes we recorded to
+#include "fweelin_block.h"
+
+// Number of beats per bar 
+// (used only to compute sync, since FW does not use the concept of bars/beats internally)
+#define SYNC_BEATS_PER_BAR 4
+
+// Maximum number of time pulses
+#define MAX_PULSES 10 
+
+// How many different sets of loop selections to remember?
+#define NUM_LOOP_SELECTION_SETS 10 
+
+// Keep track of the last n indexes we recorded to
+#define LAST_REC_COUNT 8 
 
 // Divisions are added in browsers wherever files are greater than
 // FWEELIN_FILE_BROWSER_DIVISION_TIME seconds apart
@@ -22,13 +34,24 @@
 #define FWEELIN_OUTPUT_TIMING_EXT  ".wav.usx"
 #define FWEELIN_OUTPUT_LOOP_NAME   "loop"
 #define FWEELIN_OUTPUT_SCENE_NAME  "scene"
-#define FWEELIN_OUTPUT_AUDIO_EXT   ".ogg"
 #define FWEELIN_OUTPUT_DATA_EXT    ".xml"
 
 // Console sequence for error color
 #define FWEELIN_ERROR_COLOR_ON "\033[31;1m"
 #define FWEELIN_ERROR_COLOR_OFF "\033[0m"
 
+#ifdef __MACOSX__
+// On Linux, FWEELIN_DATADIR refers to /usr/local/share/fweelin as set by autoconf
+// On Mac, we store our data in the Resource directory of the fweelin.app bundle. 
+// This string stores the location of the Resource directory, and is set in the Obj-C stub
+// for the code.
+extern char *FWEELIN_DATADIR;
+
+// On Linux, VERSION is defined within configure.ac
+// For Mac OS, we set it here:
+#define VERSION "0.5.5"
+#endif
+
 #include "fweelin_datatypes.h"
 #include "fweelin_audioio.h"
 #include "fweelin_event.h"
@@ -696,10 +719,13 @@ class FloConfig {
   // Event hook- right now, we only support one at a time
   EventHook *ev_hook;
 
-  // Extracts a comma separated array of floats from the given string-
-  // returns size of array in 'size'
-  float *ExtractArray(char *n, int *size);
-
+  // Extracts an array of floats (delimited by character delim_char)
+  // from the given string- returns size of array in 'size'
+  float *ExtractArray(char *n, int *size, char delim_char = ',');
+  
+  // Same, with ints
+  int *ExtractArrayInt(char *n, int *size, char delim_char = ',');
+  
   // Library path
   inline char *GetLibraryPath() { 
     if (librarypath != 0)
@@ -715,6 +741,11 @@ class FloConfig {
   inline int GetNumMIDIOuts() { return midiouts; };
   int midiouts;
 
+  // List of MIDI ports to transmit sync info to
+  inline int GetNumMIDISyncOuts() { return msnumouts; };
+  inline int *GetMIDISyncOuts() { return msouts; };
+  int msnumouts, *msouts;
+  
   // Is input/output #n stereo?
   inline char IsStereoInput(int n) { return ms_inputs[n]; };
   inline char IsStereoOutput(int n) { return IsStereoMaster(); };
@@ -744,6 +775,46 @@ class FloConfig {
   // Limiter release rate
   inline float GetLimiterReleaseRate() { return limiterreleaserate; };
   float limiterreleaserate;
+  
+  // File format to save loops to
+  inline codec GetLoopOutFormat() { return loopoutformat; };
+  codec loopoutformat;
+
+  // File format to save streams to
+  inline codec GetStreamOutFormat() { return streamoutformat; }; 
+  codec streamoutformat;
+
+  inline char *GetCodecName (codec i) {
+    switch (i) {
+      case VORBIS: return "ogg"; 
+      case WAV: return "wav"; 
+      case FLAC: return "flac";
+      case AU: return "au";
+      default: return "UNKNOWN";
+    }
+  };
+
+  inline codec GetCodecFromName (const char *n) {
+    for (codec i = FIRST_FORMAT; i < END_OF_FORMATS; i = (codec) (i+1))
+      if (!strcasecmp(n,GetCodecName(i)))
+	return i;
+    
+    return UNKNOWN;
+  };
+
+  inline char *GetAudioFileExt (codec i) {
+    switch (i) { 
+      case VORBIS: return ".ogg"; 
+      case WAV: return ".wav"; 
+      case FLAC: return ".flac";
+      case AU: return ".au";
+      default: return ".ogg";
+    };
+  };
+
+  // Quality for encoding OGG files
+  inline float GetVorbisEncodeQuality() { return vorbis_encode_quality; };
+  float vorbis_encode_quality;
 
   // Number of triggers (loop ids)
   inline int GetNumTriggers() { return num_triggers; };
