@@ -8,7 +8,11 @@
    ********
 */
 
+#ifdef __MACOSX__
+#include <openssl/md5.h>
+#else
 #include <gnutls/openssl.h>
+#endif
 
 #include "fweelin_midiio.h"
 #include "fweelin_videoio.h"
@@ -32,7 +36,7 @@ class AudioBlockIterator;
 class Loop;
 class Pulse;
 class BED_MarkerPoints;
-class VorbisStreamer;
+class FileStreamer;
 class PreallocatedType;
 class AudioBuffers;
 class InputSettings;
@@ -172,12 +176,12 @@ class Loop : public Saveable {
   friend class LoopManager;
 
 public:
-  const static float MIN_VOL = 0.01;
+  const static float MIN_VOL;
 
   Loop (AudioBlock *blocks, Pulse *pulse, float quant, float vol, 
-	long nbeats) : 
-    name(0), blocks(blocks), pulse(pulse), quant(quant), 
-    vol(vol), dvol(1.0), nbeats(nbeats) {};
+	long nbeats, codec format) : 
+    name(0), format(format), blocks(blocks), pulse(pulse), quant(quant), 
+    vol(vol), dvol(1.0), nbeats(nbeats), selcnt(0) {};
   virtual ~Loop () {
     if (name != 0)
       delete[] name; 
@@ -192,15 +196,22 @@ public:
       // Apply delta
       if (dvol > 1.0 && vol < MIN_VOL)
 	vol = MIN_VOL;
-      vol *=dvol;
+      vol *= dvol;
     }
   };
 
+  inline void ChangeSelectedCount (int delta) {
+    selcnt += delta;
+    if (selcnt < 0)
+      selcnt = 0;
+  };
+  
   // Perhaps store the length of the loop...
   // Because right now GetTotalLength is a calculation
   // in AudioBlock
 
   char *name;         // Name of loop, or null
+  codec format;       // File format on disk
   AudioBlock *blocks; // Chain of blocks that form this loop
   Pulse *pulse;       // Time pulse to which this loop is quantized
   float quant;        // Quantization factor
@@ -209,6 +220,81 @@ public:
     dvol;             // Rate of volume change
 
   long nbeats;        // Number of beats in this loop
+  int selcnt;         // Number of times this loop is currently selected
+                      // (see toggle-select-loop). A loop can be selected
+                      // in several sets
+};
+
+// A set of loops
+class LoopList {
+public:
+  LoopList (int l_idx = -1) : l_idx(l_idx), next(0) {};
+
+  // Scans through the list beginning with 'first' for the loop 'l_idx'-
+  // returns the LoopList containing 'item', and sets 'prev' to the previous
+  // LoopList. If not found, returns 0.
+  static LoopList *Scan (LoopList *first, int l_idx, LoopList **prev) {
+    LoopList *cur = first,
+      *tprev = 0;
+    while (cur != 0 && cur->l_idx != l_idx) {
+      tprev = cur;
+      cur = cur->next;
+    }
+
+    *prev = tprev;
+    return cur;
+  };
+
+  // Add to the beginning of a list of loops- return the new first pointer
+  static LoopList *AddBegin (LoopList *first, int l_idx) {
+    if (first == 0)
+      return new LoopList(l_idx);
+    else {
+      LoopList *nwl = new LoopList(l_idx);
+      nwl->next = first;
+      return nwl;
+    }
+  };
+
+  // Remove the given looplist 'll' from the list of loops- 
+  // you must also pass the element previous to ll
+  // We return the new first pointer
+  static LoopList *Remove (LoopList *first, LoopList *ll, LoopList *prev) {
+    if (ll != 0) {
+      if (ll == first) {
+	LoopList *nwfirst = ll->next;
+	delete ll;
+	return nwfirst; 
+      } else {
+	if (prev != 0) 
+	  prev->next = ll->next;
+	delete ll;
+	return first;
+      }
+    }
+
+    return first;
+  };
+
+  // Remove the given loop from the list of loops- return the new first pointer
+  static LoopList *Remove (LoopList *first, int l_idx) {
+    LoopList *prev,
+      *ll = Scan(first,l_idx,&prev);
+    return Remove(first,ll,prev);
+  };
+
+  // Erase a list of loops, given the first
+  static void Delete (LoopList *first) {
+    LoopList *cur = first;
+    while (cur != 0) {
+      LoopList *tmp = cur->next;
+      delete cur;
+      cur = tmp;
+    }
+  };
+
+  int l_idx;
+  LoopList *next;
 };
 
 // This class manages sets of loops, allowing them to be triggered..
@@ -226,12 +312,13 @@ public:
     delete[] map;
   }
 
-  // Save scene
-  virtual void Save(Fweelin *app);
+  // Save a whole scene, with an optional filename-
+  // if none is given, saves a new scene
+  virtual void Save(Fweelin *app, char *filename = 0);
 
   // Go save- when all loops are saved, this part saves the XML data
   // for the scene
-  void GoSave();
+  void GoSave(char *filename);
 
   inline void SetMap (int index, Loop *smp);
 
@@ -548,11 +635,17 @@ public:
   // Delete the loop at the specified index..
   void DeleteLoop (int index);
 
+  // Remove the given loop index from all loop lists
+  void UpdateLoopLists_ItemRemoved (int l_idx);
+
+  // Update all loop lists to reflect that a loop has moved in the TriggerMap
+  void UpdateLoopLists_ItemMoved (int l_idx_old, int l_idx_new);
+
   // Trigger the loop at index within the map
   // The exact behavior varies depending on what is already happening with
   // this loop and the settings passed- see ~/.fweelin/.fweelin.rc
   void Activate (int index, char shot = 0, float vol = 1.0, nframes_t ofs = 0,
-		 char overdub = 0, float od_feedback = 0.5);
+		 char overdub = 0, float *od_feedback = 0);
 
   void Deactivate (int index);
 
@@ -561,10 +654,12 @@ public:
   // Load loop from disk into the given index
   void LoadLoop(char *filename, int index, float vol = 1.0);
 
-  // Saves a scene of all loops
-  void SaveScene();
+  // Saves a new scene of all loops
+  void SaveNewScene();
+  // Saves over the current scene
+  void SaveCurScene();
   // Load scene from disk
-  void LoadScene(char *filename);
+  void LoadScene(SceneBrowserItem *i);
 
   // We receive calls periodically for saving of loops-
   // here, we return blocks to save from loops which need saving
@@ -580,7 +675,7 @@ public:
 
   // Adds the loop/scene with given filename to the browser br
   void AddLoopToBrowser(Browser *br, char *filename);
-  void AddSceneToBrowser(Browser *br, char *filename);
+  SceneBrowserItem *AddSceneToBrowser(Browser *br, char *filename);
 
   inline void SetAutoLoopSaving(char save) { autosave = save; };
   void AddToSaveQueue(Event *ev);
@@ -663,7 +758,7 @@ public:
   float *waitactivate_vol;
   char *waitactivate_od,
     *waitactivate_shot;
-  float *waitactivate_od_fb;
+  float **waitactivate_od_fb;
   LoopStatus *status; // For each index, what's the status?
 
   // Block managers that load/save loops
@@ -689,7 +784,6 @@ public:
 // This is Fweelin
 class Fweelin : public EventProducer, public BrowserCallback {
  public:
-
   Fweelin() : 
 
 #if USE_FLUIDSYNTH
@@ -699,7 +793,7 @@ class Fweelin : public EventProducer, public BrowserCallback {
     mmg(0), bmg(0), emg(0), rp(0), tmap(0), 
     loopmgr(0), browsers(0), abufs(0), iset(0), audio(0), midi(0), sdlio(0), 
     vid(0), scope(0), scope_len(0), audiomem(0), amrec(0),  
-    running(0) {};
+    sync_type(0), sync_speed(1), running(0) {};
   ~Fweelin() {};
 
   char IsRunning() { return running; };
@@ -710,7 +804,7 @@ class Fweelin : public EventProducer, public BrowserCallback {
   int go();
 
   void ToggleDiskOutput();
-  void FlushWriteName() { strcpy(writename,""); };
+  void FlushStreamOutName() { strcpy(streamoutname,""); };
 
   inline nframes_t getBUFSZ() { return fragmentsize; };
   inline sample_t *getSCOPE() { return scope; };
@@ -734,7 +828,7 @@ class Fweelin : public EventProducer, public BrowserCallback {
   inline BlockManager *getBMG() { return bmg; };
   inline EventManager *getEMG() { return emg; };
   inline RootProcessor *getRP() { return rp; };
-  inline VorbisStreamer *getSTREAMER() { return vs; };
+  inline FileStreamer *getSTREAMER() { return fs; };
   inline TriggerMap *getTMAP() { return tmap; };
   inline LoopManager *getLOOPMGR() { return loopmgr; };
   
@@ -745,7 +839,10 @@ class Fweelin : public EventProducer, public BrowserCallback {
 
   inline FloConfig *getCFG() { return cfg; };
 
-  inline char *getOUTPUTNAME() { return writename; };
+  inline char *getSTREAMOUTNAME() { return streamoutname; };
+
+  inline SceneBrowserItem *getCURSCENE() { return curscene; };
+  inline void setCURSCENE(SceneBrowserItem *nw) { curscene = nw; };
 
   inline PreallocatedType *getPRE_EXTRACHANNEL() { return pre_extrachannel; };
   inline PreallocatedType *getPRE_AUDIOBLOCK() { return pre_audioblock; };
@@ -761,6 +858,26 @@ class Fweelin : public EventProducer, public BrowserCallback {
       return 0;
   }
 
+  inline LoopList **getLOOPSEL(int idx) {
+    if (idx >= 0 && idx < NUM_LOOP_SELECTION_SETS) 
+      return &loopsel[idx];
+    else
+      return 0;
+  };
+
+  // Sync parameters
+  
+  inline int GetSyncSpeed() { return sync_speed; };
+  inline char GetSyncType() { return sync_type; };
+  
+  inline void SetSyncSpeed(int sspd) { 
+    if (sspd < 1)
+      sync_speed = 1;
+    else
+      sync_speed = sspd; 
+  };
+  inline void SetSyncType(char stype) { sync_type = stype; };
+    
   // Patch browser callbacks
   virtual void ItemBrowsed(BrowserItem *i) { ItemSelected(i); };
   virtual void ItemSelected(BrowserItem *i);
@@ -784,11 +901,17 @@ class Fweelin : public EventProducer, public BrowserCallback {
     *pre_extrachannel,
     *pre_timemarker;
 
-  // ****************** VORBIS DISK STREAMER
-  VorbisStreamer *vs;
-  int writenum; // Number of ogg file currently being written
-  char writename[FWEELIN_OUTNAME_LEN], // Name of ogg file
-    timingname[FWEELIN_OUTNAME_LEN]; // Name of timing stripe file
+  // ******************  DISK STREAMER
+  FileStreamer *fs;
+  int writenum; // Number of audio output file currently being written
+  char streamoutname[FWEELIN_OUTNAME_LEN], // Name of output file
+    timingname[FWEELIN_OUTNAME_LEN];       // Name of timing stripe file
+
+  // ******************  LOOP SELECTIONS
+  LoopList *loopsel[NUM_LOOP_SELECTION_SETS];
+
+  // ******************  SCENE INFO
+  SceneBrowserItem *curscene;
 
   // ****************** SYSTEM LEVEL AUDIO  
   // Audio buffers
@@ -821,6 +944,11 @@ class Fweelin : public EventProducer, public BrowserCallback {
   // Control settings 
   FloConfig *cfg;
   
+  // Variables for sync
+  
+  char sync_type; // Nonzero for beat-sync and zero for bar-sync
+  int sync_speed; // Number of beats or bars (external) per pulse (FW)
+  
   char running; // Nonzero if FW is fully started
 };
 
