@@ -5,8 +5,13 @@
 #include "fweelin_event.h"
 #include "fweelin_block.h"
 
+#ifndef MIN
 #define MIN(a,b) ((a) < (b) ? (a) : (b))
+#endif
+
+#ifndef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
+#endif
 
 class Fweelin;
 class Loop;
@@ -291,6 +296,9 @@ public:
   inline char IsMetronomeActive() { return metroactive; };
   inline void SwitchMetronome(char active) { metroactive = active; };
 
+  // Start/stop sending MIDI clock for this pulse
+  void SetMIDIClock (char start);
+  
   // Quantizes src length to fit to this pulse length 
   nframes_t QuantizeLength(nframes_t src);
 
@@ -396,6 +404,8 @@ public:
 
   nframes_t syncpos[MAX_SYNC_POS]; // Sync positions
   int numsyncpos; // Current number of sync positions
+  
+  SyncStateType clockrun;  // Status of MIDI clock
 };
 
 // This is the base of signal processing tree- it connects to system level audio
@@ -489,7 +499,8 @@ public:
   // Extra tail on record end facilitates smooth crossfade for
   // sync-recorded loops
   const static nframes_t REC_TAIL_LEN = 1024;
-
+  const static float OVERDUB_DEFAULT_FEEDBACK;
+  
   // Notes:
  
   // One caveat, since hipri mgrs have granularity of only one fragment
@@ -513,7 +524,7 @@ public:
 		  Loop *od_loop,
 		  float od_playvol,
 		  nframes_t od_startofs,
-		  float od_feedback);
+		  float *od_feedback);	// Pointer to value for feedback (can be continuously varied)
 
   // Recording new blocks, growing size as necessary	       
   RecordProcessor(Fweelin *app,
@@ -562,15 +573,15 @@ public:
   void FadeOut_Input(nframes_t len, 
 		     sample_t *input_l, sample_t *input_r,
 		     sample_t *loop_l, sample_t *loop_r, 
-		     float feedback, 
+		     float old_fb, float new_fb, float fb_delta,
 		     sample_t *dest_l, sample_t *dest_r);
 
   // Fades input samples into mix- writing to 'dest'
   void FadeIn_Input(nframes_t len, 
-		     sample_t *input_l, sample_t *input_r,
-		     sample_t *loop_l, sample_t *loop_r, 
-		     float feedback, 
-		     sample_t *dest_l, sample_t *dest_r);
+		    sample_t *input_l, sample_t *input_r,
+		    sample_t *loop_l, sample_t *loop_r, 
+		    float old_fb, float new_fb, float fb_delta,
+		    sample_t *dest_l, sample_t *dest_r);
 
   // Jumps to a position within an overdubbing loop- fade of input & output
   void Jump(nframes_t ofs);
@@ -623,7 +634,9 @@ public:
   // Overdub settings
   Loop *od_loop;
   float od_playvol, 
-    od_feedback;
+    *od_feedback,
+    od_feedback_lastval;  // Last value for feedback- used to determine delta, to remove
+			  // zipper noise
   long od_curbeat; // Current beat in od_loop
   char od_fadein, od_fadeout, od_stop, // Fade in overdub, fade out overdub,
                                        // and stop overdub flags
@@ -682,25 +695,27 @@ public:
   long curbeat;
 };
 
-class VorbisStreamer : public Processor, public EventListener {
+class FileStreamer : public Processor, public EventListener {
  public:
   const static nframes_t OUTPUTBUFLEN = 100000;
   const static int MARKERBUFLEN = 50;
 
-  VorbisStreamer(Fweelin *app, nframes_t outbuflen = OUTPUTBUFLEN);
-  virtual ~VorbisStreamer();
+  FileStreamer(Fweelin *app, nframes_t outbuflen = OUTPUTBUFLEN);
+  virtual ~FileStreamer();
 
   virtual void process(char pre, nframes_t len, AudioBuffers *ab);
   virtual void ReceiveEvent(Event *ev, EventProducer *from);
   // virtual int ProcessorCall(void *trigger, int data);
 
-  // Starts writing to a new vorbis audio stream
+  // Starts writing to a new audio stream
   // Note all the heavy work is done in the encode thread!
-  int StartWriting(char *filename,char *timingname) {
+  int StartWriting(char *filename, char *timingname, codec type) {
     if (outname != 0) 
       return -1; // Already writing!
 
     outname = filename;
+    outputsize = 0;
+    filetype = type;
     this->timingname = timingname;
     writerstatus = STATUS_START_PENDING;
 
@@ -714,6 +729,9 @@ class VorbisStreamer : public Processor, public EventListener {
   };
 
   char GetStatus() { return writerstatus; };
+
+  // Returns the number of *frames* written so far-- not bytes
+  // The actual file size will vary depending on the codec used
   long int GetOutputSize() { return outputsize; };
 
   static void *run_encode_thread (void *ptr);
@@ -731,21 +749,22 @@ class VorbisStreamer : public Processor, public EventListener {
 
   // Encoding in stereo?
   char stereo;
+  codec filetype;
 
   // File
-  FILE *outfd,   // Current output filedescriptor (OGG)
-    *timingfd;   // Current timing output filedescriptor (USX)
+  FILE *outfd,   // Current output filedescriptor (audio)
+    *timingfd;   // Current timing output filedescriptor (data- USX)
   char *outname, // Current output filename
     *timingname; // Current timing output filename
 
-  // Time markers for storing downbeat points along with OGG
+  // Time markers for storing downbeat points along with audio
   TimeMarker *marks;
   int mkwriteidx,
     mkreadidx;
 
-  // Number of beat triggers passed in this vorbis recording 
+  // Number of beat triggers passed in this recording 
   long nbeats;
-  // Global sample count at start of OGG file
+  // Global sample count at start of stream file
   nframes_t startcnt;
 
   // Output buffers
@@ -758,12 +777,12 @@ class VorbisStreamer : public Processor, public EventListener {
   // Number of bytes written to output file
   long int outputsize;
 
-  // Vorbis encode/disk write thread
+  // Disk encode/disk write thread
   pthread_t encode_thread;
   int threadgo;
 
-  // Vorbis encoder
-  VorbisEncoder *enc;
+  // Encoder
+  iFileEncoder *enc;
 };
 
 // PassthroughProcessor creates a monitor mix of all inputs into the given output
