@@ -81,10 +81,13 @@ enum EventType {
   T_EV_Input_MIDIController,
   T_EV_Input_MIDIProgramChange,
   T_EV_Input_MIDIPitchBend,
+  T_EV_Input_MIDIClock,
+  T_EV_Input_MIDIStartStop,
   T_EV_StartSession,
   T_EV_GoSub,
 
   T_EV_LoopClicked,
+  T_EV_BrowserItemBrowsed,
 
   // End of bindable events-
   // Events after this can not trigger config bindings
@@ -131,6 +134,14 @@ enum EventType {
   T_EV_SwitchMetronome,
   T_EV_SetSyncType,
   T_EV_SetSyncSpeed,
+  T_EV_SetMidiSync,
+  
+  T_EV_ToggleSelectLoop,
+  T_EV_SelectOnlyPlayingLoops,
+  T_EV_SelectAllLoops,
+  T_EV_TriggerSelectedLoops,
+  T_EV_SetSelectedLoopsTriggerVolume,
+  T_EV_InvertSelection,
 
   T_EV_SetTriggerVolume,
   T_EV_SlideLoopAmp,
@@ -146,7 +157,8 @@ enum EventType {
   T_EV_ToggleDiskOutput,
   T_EV_SetAutoLoopSaving,
   T_EV_SaveLoop,
-  T_EV_SaveScene,
+  T_EV_SaveNewScene,
+  T_EV_SaveCurrentScene,
   T_EV_SetLoadLoopId,
   T_EV_SetDefaultLoopPlacement,
 
@@ -155,6 +167,7 @@ enum EventType {
   T_EV_BrowserSelectItem,
   T_EV_BrowserRenameItem,
   T_EV_PatchBrowserMoveToBank,
+  T_EV_PatchBrowserMoveToBankByIndex,
 
   T_EV_Last
 };
@@ -193,8 +206,9 @@ class EventParameter {
 class EventTypeTable {
  public:
   EventTypeTable (char *name = 0, PreallocatedType *mgr = 0,
-		  Event *proto = 0, int paramidx = -1) :
-    name(name), mgr(mgr), proto(proto), paramidx(paramidx) {};
+		  Event *proto = 0, int paramidx = -1, char slowdelivery = 0) :
+    name(name), mgr(mgr), proto(proto), paramidx(paramidx), 
+    slowdelivery(slowdelivery) {};
 
   char *name;
   PreallocatedType *mgr;
@@ -202,6 +216,9 @@ class EventTypeTable {
   int paramidx; // Index of event parameter that is used for hash index
                 // For example, in the keyboard input event, the keysym 
                 // is indexed for quick triggering of keyboard events by key
+  char slowdelivery; // Nonzero if this event should be delivered slow, in
+                     // a nonRT thread. This is useful for events that cause
+                     // nonRT-safe operations to be executed.
 };
 
 // Events can be allocated in realtime using class Preallocated
@@ -215,6 +232,7 @@ class Event : public Preallocated {
     to = 0;
     next = 0;
     time = 0;
+    echo = 0;
   };
 
   virtual void operator = (const Event &src) {};
@@ -254,6 +272,8 @@ class Event : public Preallocated {
   Event *next;
   // Timing of event (s)
   double time;
+  // Event is an echo of an unhandled event?
+  char echo;
 };
 
 // GoSub is really an event that encapsulates other events
@@ -322,6 +342,7 @@ class KeyInputEvent : public Event {
   char down; // Nonzero if key is pressed, zero if key is released
   int keysym, // Keysym of key pressed
     unicode;  // Unicode translation (if enabled)
+  int presslen;
 };
 
 class LoopClickedEvent : public Event {
@@ -354,6 +375,7 @@ class LoopClickedEvent : public Event {
     in;      // Zero if clicked in looptray, one if clicked in layout
   int button; // Button # pressed/released
   int loopid; // LoopID of loop that was clicked
+  int presslen;
 };
 
 class JoystickButtonInputEvent : public Event {
@@ -382,6 +404,7 @@ class JoystickButtonInputEvent : public Event {
   char down;    // Nonzero if button is pressed, zero if button is released
   int button;   // Button # pressed/released
   int joystick; // Index of joystick
+  int presslen;
 };
 
 class MouseButtonInputEvent : public Event {
@@ -413,6 +436,7 @@ class MouseButtonInputEvent : public Event {
   char down; // Nonzero if button is pressed, zero if button is released
   int button; // Button # pressed/released
   int x, y;   // Coordinates of press (on screen)
+  int presslen;
 };
 
 class MouseMotionInputEvent : public Event {
@@ -583,29 +607,97 @@ class MIDIKeyInputEvent : public Event {
     vel;       // velocity
 };
 
+class MIDIClockInputEvent : public Event {
+public:
+  EVT_DEFINE(MIDIClockInputEvent,T_EV_Input_MIDIClock);
+  virtual void Recycle() {
+    outport = 1;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    MIDIClockInputEvent &s = (MIDIClockInputEvent &) src;
+    outport = s.outport;
+  };
+  virtual int GetNumParams() { return 1; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+      case 0:
+	return EventParameter("outport",FWEELIN_GETOFS(outport),T_int);
+    }
+    
+    return EventParameter();
+  };    
+  
+  int outport; // # of MIDI output to send event to
+};
+
+class MIDIStartStopInputEvent : public Event {
+public:
+  EVT_DEFINE(MIDIStartStopInputEvent,T_EV_Input_MIDIStartStop);
+  virtual void Recycle() {
+    outport = 1;
+    start = 0;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    MIDIStartStopInputEvent &s = (MIDIStartStopInputEvent &) src;
+    outport = s.outport;
+    start = s.start;
+  };
+  virtual int GetNumParams() { return 2; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+      case 0:
+	return EventParameter("outport",FWEELIN_GETOFS(outport),T_int);
+      case 1:
+	return EventParameter("start",FWEELIN_GETOFS(start),T_char);
+    }
+    
+    return EventParameter();
+  };    
+  
+  int outport; // # of MIDI output to send event to
+  char start;  // 1- MIDI Start, 0- MIDI Stop
+};
+
 class SetVariableEvent : public Event {
  public:
   EVT_DEFINE(SetVariableEvent,T_EV_SetVariable);
+  virtual void Recycle() {
+    maxjumpcheck = 0;
+    Event::Recycle();
+  };
   virtual void operator = (const Event &src) {
     SetVariableEvent &s = (SetVariableEvent &) src;
     var = s.var;
     value.type = s.value.type;
     value = s.value;
+    maxjumpcheck = s.maxjumpcheck;
+    maxjump.type = s.maxjump.type;
+    maxjump = s.maxjump;
   };
-  virtual int GetNumParams() { return 2; };
+  virtual int GetNumParams() { return 4; };
   virtual EventParameter GetParam(int n) { 
     switch (n) {
     case 0:
       return EventParameter("var",FWEELIN_GETOFS(var),T_variableref);
     case 1:
       return EventParameter("value",FWEELIN_GETOFS(value),T_variable);      
+    case 2:
+      return EventParameter("maxjumpcheck",FWEELIN_GETOFS(maxjumpcheck),T_char);      
+    case 3:
+      return EventParameter("maxjump",FWEELIN_GETOFS(maxjump),T_variable);      
     }
 
     return EventParameter();
   };    
 
   UserVariable *var;  // Variable to set
-  UserVariable value; // Value to set it to
+  UserVariable value, // Value to set it to
+    maxjump;	      // Maximum jump in variable between current value and new value
+		      // Jumps beyond maxjump cause the variable not to be set
+  char maxjumpcheck;  // Nonzero if we should check variable change against maxjump-
+		      // If maxjumpcheck is zero, the variable is always set
 };
 
 class ToggleVariableEvent : public Event {
@@ -1168,7 +1260,7 @@ class TriggerLoopEvent : public Event {
     engage = -1;
     shot = 0;
     od = 0;
-    od_fb = 0.5;
+    od_fb = 0;
     Event::Recycle();
   };
   virtual void operator = (const Event &src) {
@@ -1194,7 +1286,7 @@ class TriggerLoopEvent : public Event {
     case 4:
       return EventParameter("overdub",FWEELIN_GETOFS(od),T_char);
     case 5:
-      return EventParameter("overdubfeedback",FWEELIN_GETOFS(od_fb),T_float);
+      return EventParameter("overdubfeedback",FWEELIN_GETOFS(od_fb),T_variableref);
     }
 
     return EventParameter();
@@ -1208,7 +1300,7 @@ class TriggerLoopEvent : public Event {
   char shot;   // Nonzero if we should only play thru once- no loop
   float vol;   // Volume of trigger
   char od;     // Nonzero if we should trigger an overdub
-  float od_fb; // Overdub feedback
+  UserVariable *od_fb;  // Variable which holds overdub feedback- can be continuously varied
 };
 
 class MoveLoopEvent : public Event {
@@ -1310,9 +1402,14 @@ class SaveLoopEvent : public Event {
   int index; // Index of loop to save
 };
 
-class SaveSceneEvent : public Event {
+class SaveNewSceneEvent : public Event {
  public:
-  EVT_DEFINE(SaveSceneEvent,T_EV_SaveScene);
+  EVT_DEFINE(SaveNewSceneEvent,T_EV_SaveNewScene);
+};
+
+class SaveCurrentSceneEvent : public Event {
+public:
+  EVT_DEFINE(SaveCurrentSceneEvent,T_EV_SaveCurrentScene);
 };
 
 class SetLoadLoopIdEvent : public Event {
@@ -1472,6 +1569,30 @@ class BrowserRenameItemEvent : public Event {
   int browserid;   // Display ID of browser
 };
 
+class BrowserItemBrowsedEvent : public Event {
+ public:
+  EVT_DEFINE(BrowserItemBrowsedEvent,T_EV_BrowserItemBrowsed);
+  virtual void Recycle() {
+    browserid = -1;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    BrowserItemBrowsedEvent &s = (BrowserItemBrowsedEvent &) src;
+    browserid = s.browserid;
+  };
+  virtual int GetNumParams() { return 1; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("browserid",FWEELIN_GETOFS(browserid),T_int);
+    }
+
+    return EventParameter();
+  };    
+
+  int browserid;   // Display ID of browser
+};
+
 class PatchBrowserMoveToBankEvent : public Event {
  public:
   EVT_DEFINE(PatchBrowserMoveToBankEvent,T_EV_PatchBrowserMoveToBank);
@@ -1496,6 +1617,32 @@ class PatchBrowserMoveToBankEvent : public Event {
   int direction;   // Direction to move (+1/-1 next/previous)
 };
 
+class PatchBrowserMoveToBankByIndexEvent : public Event {
+ public:
+  EVT_DEFINE(PatchBrowserMoveToBankByIndexEvent,
+	     T_EV_PatchBrowserMoveToBankByIndex);
+  virtual void Recycle() {
+    index = 0;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    PatchBrowserMoveToBankByIndexEvent &s = 
+      (PatchBrowserMoveToBankByIndexEvent &) src;
+    index = s.index;
+  };
+  virtual int GetNumParams() { return 1; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("index",FWEELIN_GETOFS(index),T_int);
+    }
+
+    return EventParameter();
+  };    
+
+  int index; // Index of patchbank to choose
+};
+
 class RenameLoopEvent : public Event {
  public:
   EVT_DEFINE(RenameLoopEvent,T_EV_RenameLoop);
@@ -1638,7 +1785,7 @@ class SetSyncSpeedEvent : public Event {
   EVT_DEFINE(SetSyncSpeedEvent,T_EV_SetSyncSpeed);
   virtual void operator = (const Event &src) {
     SetSyncSpeedEvent &s = (SetSyncSpeedEvent &) src;
-    sspd= s.sspd;
+    sspd = s.sspd;
   };
   virtual int GetNumParams() { return 1; };
   virtual EventParameter GetParam(int n) { 
@@ -1653,6 +1800,185 @@ class SetSyncSpeedEvent : public Event {
   int sspd; // Number of external transport beats/bars per internal pulse
 };
 
+class SetMidiSyncEvent : public Event {
+public:
+  EVT_DEFINE(SetMidiSyncEvent,T_EV_SetMidiSync);
+  virtual void operator = (const Event &src) {
+    SetMidiSyncEvent &s = (SetMidiSyncEvent &) src;
+    midisync = s.midisync;
+  };
+  virtual int GetNumParams() { return 1; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+      case 0:
+	return EventParameter("midisync",FWEELIN_GETOFS(midisync),T_int);
+    }
+    
+    return EventParameter();
+  };    
+  
+  int midisync; // Nonzero to transmit MIDI sync, zero for no MIDI sync
+};
+
+class ToggleSelectLoopEvent : public Event {
+ public:
+  EVT_DEFINE(ToggleSelectLoopEvent,T_EV_ToggleSelectLoop);
+  virtual void operator = (const Event &src) {
+    ToggleSelectLoopEvent &s = (ToggleSelectLoopEvent &) src;
+    setid = s.setid;
+    loopid = s.loopid;
+  };
+  virtual int GetNumParams() { return 2; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    case 1:
+      return EventParameter("loopid",FWEELIN_GETOFS(loopid),T_int);
+    }
+
+    return EventParameter();
+  };    
+
+  int setid, // ID # of the selection set in which to toggle the loop
+    loopid;  // ID # of loop to toggle
+};
+
+class SelectOnlyPlayingLoopsEvent : public Event {
+ public:
+  EVT_DEFINE(SelectOnlyPlayingLoopsEvent,T_EV_SelectOnlyPlayingLoops);
+  virtual void operator = (const Event &src) {
+    SelectOnlyPlayingLoopsEvent &s = (SelectOnlyPlayingLoopsEvent &) src;
+    setid = s.setid;
+    playing = s.playing;
+  };
+  virtual int GetNumParams() { return 2; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    case 1:
+      return EventParameter("playing",FWEELIN_GETOFS(playing),T_char);
+    }
+
+    return EventParameter();
+  };    
+
+  int setid; // ID # of the selection set to work on
+  char playing; // Nonzero if we should select only those loops currently 
+                // playing-- zero if we should select only those loops 
+                // currently idle
+};
+
+class SelectAllLoopsEvent : public Event {
+ public:
+  EVT_DEFINE(SelectAllLoopsEvent,T_EV_SelectAllLoops);
+  virtual void operator = (const Event &src) {
+    SelectAllLoopsEvent &s = (SelectAllLoopsEvent &) src;
+    setid = s.setid;
+    select = s.select;
+  };
+  virtual int GetNumParams() { return 2; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    case 1:
+      return EventParameter("select",FWEELIN_GETOFS(select),T_char);
+    }
+
+    return EventParameter();
+  };    
+
+  int setid; // ID # of the selection set to work on
+  char select; // Nonzero to select/zero to unselect all loops
+};
+
+class InvertSelectionEvent : public Event {
+public:
+  EVT_DEFINE(InvertSelectionEvent,T_EV_InvertSelection);
+  virtual void operator = (const Event &src) {
+    InvertSelectionEvent &s = (InvertSelectionEvent &) src;
+    setid = s.setid;
+  };
+  virtual int GetNumParams() { return 1; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+      case 0:
+	return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    }
+    
+    return EventParameter();
+  };    
+  
+  int setid; // ID # of the selection set to invert (select all other loops)
+};
+
+class TriggerSelectedLoopsEvent : public Event {
+ public:
+  EVT_DEFINE(TriggerSelectedLoopsEvent,T_EV_TriggerSelectedLoops);
+  virtual void Recycle() {
+    setid = 0;
+    vol = 1.0;
+    toggleloops = 0;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    TriggerSelectedLoopsEvent &s = (TriggerSelectedLoopsEvent &) src;
+    setid = s.setid;
+    vol = s.vol;
+    toggleloops = s.toggleloops;
+  };
+  virtual int GetNumParams() { return 3; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    case 1:
+      return EventParameter("vol",FWEELIN_GETOFS(vol),T_float);
+    case 2:
+      return EventParameter("toggleloops",FWEELIN_GETOFS(toggleloops),T_char);
+    }
+
+    return EventParameter();
+  };    
+
+  int setid; // ID # of the selection set to work on
+  float vol; // Volume at which to trigger selected loops
+  char toggleloops; // Nonzero to toggle loops, zero to force active
+};
+
+class SetSelectedLoopsTriggerVolumeEvent : public Event {
+ public:
+  EVT_DEFINE(SetSelectedLoopsTriggerVolumeEvent,
+	     T_EV_SetSelectedLoopsTriggerVolume);
+  virtual void Recycle() {
+    setid = 0;
+    vol = 1.0;
+    Event::Recycle();
+  };
+  virtual void operator = (const Event &src) {
+    SetSelectedLoopsTriggerVolumeEvent &s = 
+      (SetSelectedLoopsTriggerVolumeEvent &) src;
+    setid = s.setid;
+    vol = s.vol;
+  };
+  virtual int GetNumParams() { return 2; };
+  virtual EventParameter GetParam(int n) { 
+    switch (n) {
+    case 0:
+      return EventParameter("setid",FWEELIN_GETOFS(setid),T_int);
+    case 1:
+      return EventParameter("vol",FWEELIN_GETOFS(vol),T_float);
+    }
+
+    return EventParameter();
+  };    
+
+  int setid; // ID # of the selection set to work on
+  float vol; // Trigger volume to set for loops
+};
+
 class EndRecordEvent : public Event {
  public:   
   EVT_DEFINE(EndRecordEvent,T_EV_EndRecord);
@@ -1684,6 +2010,12 @@ class LoopListEvent : public Event {
 class SceneMarkerEvent : public Event {
  public:   
   EVT_DEFINE(SceneMarkerEvent,T_EV_SceneMarker);
+  virtual void Recycle() {
+    strcpy(s_filename,"");
+    Event::Recycle();
+  };
+  
+  char s_filename[FWEELIN_OUTNAME_LEN];	// Filename of scene on disk
 };
 
 class PulseSyncEvent : public Event {
@@ -1827,21 +2159,27 @@ class EventManager {
 
   // Broadcast immediately!
   inline void BroadcastEventNow(Event *ev, 
-				EventProducer *source) {
+				EventProducer *source,
+				char allowslowdelivery = 1) {
     int evnum = (int) ev->GetType();
     
-    // Scan through the listeners to see who to call
-    EventListenerItem *cur = listeners[evnum];
-    while (cur != 0) {
-      if ((cur->eventsfrom == 0 && 
-	   (!cur->block_self_calls || source != (void *) cur->callwhom)) ||
-	  source == 0 || cur->eventsfrom == source)
-	cur->callwhom->ReceiveEvent(ev,source);
-      cur = cur->next;
+    // Check if this event is slow-delivery only
+    if (allowslowdelivery && Event::ett[evnum].slowdelivery)
+      BroadcastEvent(ev,source);
+    else {
+      // Scan through the listeners to see who to call
+      EventListenerItem *cur = listeners[evnum];
+      while (cur != 0) {
+	if ((cur->eventsfrom == 0 && 
+	     (!cur->block_self_calls || source != (void *) cur->callwhom)) ||
+	    source == 0 || cur->eventsfrom == source)
+	  cur->callwhom->ReceiveEvent(ev,source);
+	cur = cur->next;
+      }
+      
+      // This event has been broadcast.. erase it!.. use RTDelete()
+      ev->RTDelete();
     }
-
-    // This event has been broadcast.. erase it!.. use RTDelete()
-    ev->RTDelete();
   };
 
   // Broadcast through dispatch thread!
@@ -1859,7 +2197,10 @@ class EventManager {
       pthread_mutex_unlock (&dispatch_thread_lock);
     }
     else
-      printf("RTInterrupt active EventDispatchThread!\n");
+      // Priority inversion
+      printf("EVENT: WARNING: Priority inversion during event broadcast! Event '%s' may be lost!\n",Event::ett[(int) ev->GetType()].name);
+
+    // printf("EVENT: SENT: %s!\n",Event::ett[(int) ev->GetType()].name);
   };
 
   // Not RT safe, but threadsafe!
@@ -1931,8 +2272,10 @@ class EventManager {
 	//printf("Evt dispatch- dt: %2.2f ms\n",dt);
 
 	if (cur->GetMgr() == 0)
-	  printf("WARNING: Broadcast from RT nonRT event!!\n");
-	inst->BroadcastEventNow(cur,cur->from);
+	  printf("EVENT: WARNING: Broadcast from RT nonRT event!!\n");
+
+	// printf("EVENT: DISPATCH: %s!\n",Event::ett[(int) cur->GetType()].name);
+	inst->BroadcastEventNow(cur,cur->from,0); // Force delivery now
 	cur = cur->next;
       }
 
@@ -1944,11 +2287,14 @@ class EventManager {
       // someone is accessing the queue (for ex, BroadcastEvent)
       // So the dispatch thread holds for other threads. Not a big deal.
 
+      // ** Or, implement lockless ring buffer design here
+
       // Empty queue!
       inst->events = 0;
 
       // Wait for wakeup
       pthread_cond_wait (&inst->dispatch_ready, &inst->dispatch_thread_lock);
+      // printf("EVENT: WAKEUP!\n");
     }
 
     printf("Event Manager: end dispatch thread\n");
