@@ -24,11 +24,14 @@ public:
   virtual void ReceiveEvent(Event *ev, EventProducer *from);
 
   // Send a new patch to synth
-  void SendPatchChange(PatchItem *p, int channel);
+  void SendPatchChange(PatchItem *p);
+
+  // Receive MIDI events for synth
+  void ReceiveMIDIEvent(Event *ev);
 
   // Enable/disable FluidSynth- if disabled, bypasses
   // processor stage to reduce CPU usage, but leaves memory allocated
-  void SetEnable(char en) { 
+  inline void SetEnable(char en) { 
     // Preprocess audio for smoothing
     //dopreprocess();
     this->enable = en;
