@@ -2,6 +2,7 @@
    Art is the beginning.
 */
 
+#include <sys/stat.h>
 #include <sys/time.h>
 
 #include <stdio.h>
@@ -20,6 +21,10 @@
 #include "fweelin_core.h"
 #include "fweelin_logo.h"
 
+#ifdef __MACOSX__
+#define CAPITAL_FILLED_PIE
+#endif
+
 // Different versions of sdl-gfx have different naming
 #ifdef CAPITAL_FILLED_PIE
 #define FILLED_PIE filledPieRGBA
@@ -797,6 +802,12 @@ char CircularMap::Map(SDL_Surface *out, int dstx, int dsty) {
 // ******** VIDEO HANDLER
 
 int VideoIO::activate() {
+#ifdef __MACOSX__
+  // On Mac OS X, prime video in main thread
+  SetVideoMode(0);
+#endif
+  
+#if 1
   printf("VIDEO: Starting handler..\n");
 
   pthread_mutex_init (&video_thread_lock,0);
@@ -825,7 +836,7 @@ int VideoIO::activate() {
   if (pthread_setschedparam(video_thread, SCHED_OTHER, &schp) != 0)
     printf("VIDEO: Can't set thread priority, will use regular!\n");
 #endif
-
+  
   // Listen for events
   if (app->getEMG() == 0) {
     printf("VIDEO: Error: Event Manager not yet active!\n");
@@ -840,6 +851,7 @@ int VideoIO::activate() {
 
   app->getEMG()->ListenEvent(this,0,T_EV_Input_MouseButton);
   app->getEMG()->ListenEvent(this,0,T_EV_Input_MouseMotion);
+#endif
   
   return 0;
 }
@@ -1279,6 +1291,12 @@ char VideoIO::DrawLoop(LoopManager *loopmgr, int i,
     cursorclr = { 0xEF, 0x11, 0x11, 0 },
     txtclr2 = { 0xEF, 0xAF, 0xFF, 0 };
 
+  // Color for selected loops
+  static SDL_Color selcolor[4] = { { 0xF9, 0xE6, 0x13, 0 }, 
+				   { 0x62, 0x62, 0x62, 0 },
+				   { 0xFF, 0xFF, 0xFF, 0 },
+				   { 0xE0, 0xDA, 0xD5, 0 } };
+  
   const float cpeak_mul = 2.0, // For pulsing loops - magnitude of pulse
     cpeak_base = 0.5; // For pulsing loops - base size of loop 
   const int lscope_maxmag = OCY(15),
@@ -1320,6 +1338,12 @@ char VideoIO::DrawLoop(LoopManager *loopmgr, int i,
       plen = pa->peaks->GetTotalLen();
   }
   
+  char selected = 0;
+  if (l != 0 && l->selcnt > 0) {
+    selected = 1;
+    loopcolors = selcolor; // Color loop selected
+  }
+
   // Draw peaks & avgs
   float loopvol = 1.0, // Loop volume
     loopdvol = 1.0; // Loop volume delta
@@ -1791,6 +1815,12 @@ void VideoIO::video_event_loop ()
   numhelppages++; // 1 is considered first help page
   helpx2 = helpx + helpmaxcol1 + helpmaxcol2;
 
+  const static int CHECK_SIZE_FRAMES = 20;
+  int checksizecnt = CHECK_SIZE_FRAMES;
+  nframes_t streamoutsize = 0;
+  char streamoutstatfailed = 0;
+  double streamoutfloatsize = 0.0;
+
   video_time = 0;
   double video_start = mygettime();
   while (videothreadgo) {
@@ -1842,10 +1872,23 @@ void VideoIO::video_event_loop ()
 	  }
 
 	  SDL_Color elclr;
-	  elclr.r = (int) (loopcolors[clrnum][0].r*colormag);
-	  elclr.g = (int) (loopcolors[clrnum][0].g*colormag);
-	  elclr.b = (int) (loopcolors[clrnum][0].b*colormag);
-	  elclr.unused = 0;
+	  Loop *l = loopmgr->GetSlot(i);
+	  if (l != 0 && l->selcnt > 0) {
+	    // Selected loop
+ 	    static SDL_Color selcolor[4] = { { 0xF9, 0xE6, 0x13, 0 }, 
+					     { 0x62, 0x62, 0x62, 0 },
+					     { 0xFF, 0xFF, 0xFF, 0 },
+					     { 0xE0, 0xDA, 0xD5, 0 } };
+	    elclr.r = selcolor[0].r;
+	    elclr.g = selcolor[0].g;
+	    elclr.b = selcolor[0].b;
+	    elclr.unused = 0;
+	  } else {
+	    elclr.r = (int) (loopcolors[clrnum][0].r*colormag);
+	    elclr.g = (int) (loopcolors[clrnum][0].g*colormag);
+	    elclr.b = (int) (loopcolors[clrnum][0].b*colormag);
+	    elclr.unused = 0;
+	  }
 
 	  // Draw each geometry of this element
 	  FloLayoutElementGeometry *curgeo = curel->geo;
@@ -1902,7 +1945,7 @@ void VideoIO::video_event_loop ()
     }
 
     // Draw input scope
-#if 1
+#if 0
     peakbuf = app->getAMPEAKS()->buf,
     avgbuf = app->getAMAVGS()->buf;
     int midpt = iscopey;
@@ -1975,15 +2018,45 @@ void VideoIO::video_event_loop ()
     }
 #endif
 
-    // File output status
-    char *writename = app->getOUTPUTNAME();
-    if (strlen(writename) == 0)
+    // ** These two hardcoded displays could be made configurable
+
+    // Stream output status
+    char *writename = app->getSTREAMOUTNAME();
+    if (strlen(writename) == 0) {
       strcpy(tmp,"stream off"); // No output
-    else
-      sprintf(tmp,"%s   %lu",
-	      writename,(long int) app->getSTREAMER()->GetOutputSize());
+      checksizecnt = CHECK_SIZE_FRAMES;
+    } else {
+      if (++checksizecnt >= CHECK_SIZE_FRAMES) {
+	// Stat for size of file
+	checksizecnt = 0;
+	
+	struct stat st;
+	if (stat(app->getSTREAMOUTNAME(),&st) == 0) {
+	  streamoutstatfailed = 0;
+	  streamoutsize = st.st_size;
+	  streamoutfloatsize = streamoutsize / (1024.*1024);
+	} else {
+	  streamoutstatfailed = 1;
+	  streamoutsize = app->getSTREAMER()->GetOutputSize();
+	}
+      }
+
+      if (streamoutstatfailed) 
+	sprintf(tmp,"%s   %d frames",
+		writename,streamoutsize);
+      else
+	sprintf(tmp,"%s   %.1f mb",
+		writename,streamoutfloatsize);
+    }
     draw_text(screen,mainfont,tmp,patchx,patchy-OCY(22),gray);
-    
+
+    // Scene name
+    SceneBrowserItem *curscene = app->getCURSCENE();
+    if (curscene != 0)
+      draw_text(screen,mainfont,curscene->name,0,0,gray);
+
+    // **
+
     // Draw displays
 #if 1
     FloDisplay *curdisplay = fs->GetDisplays();
@@ -2224,7 +2297,7 @@ void VideoIO::SetVideoMode(char fullscreen) {
 
   // Disabled (slower) options:
   /*| SDL_SRCALPHA | SDL_RESIZABLE |*/
-  videoflags = SDL_HWSURFACE;
+  videoflags = SDL_HWSURFACE | SDL_DOUBLEBUF;
   this->fullscreen = fullscreen;
   if (fullscreen) 
     videoflags |= SDL_FULLSCREEN;
@@ -2253,6 +2326,10 @@ void *VideoIO::run_video_thread(void *ptr)
 
   printf("VIDEO: Thread start..\n");
 
+#ifdef __MACOSX__
+  inst->cocoa.SetupCocoaThread();
+#endif
+  
 #if 0
   // This init was moved to fweelin_core because SDL is required
   // for keyboard and config file reading as well..
@@ -2331,9 +2408,12 @@ void *VideoIO::run_video_thread(void *ptr)
   while (!inst->app->IsRunning())
     usleep(10000);
 
+#ifndef __MACOSX__
+  // On Mac OS X, this is done in the main thread
   // Set video mode / window size
   inst->SetVideoMode(0);
-
+#endif
+  
   // Start main loop
   inst->video_event_loop();
 
@@ -2350,6 +2430,10 @@ void *VideoIO::run_video_thread(void *ptr)
   // Close things up
   SDL_QuitSubSystem(SDL_INIT_VIDEO);
 
+#ifdef __MACOSX__
+  inst->cocoa.TakedownCocoaThread();
+#endif
+  
   printf("VIDEO: thread done\n");
   return 0;
 }
